/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: WizardDialog.java,v 1.4 2000/02/15 01:27:36 pmahar Exp $
 */

package org.enhydra.devsys.wizard.common;

import org.enhydra.devsys.swing.SwingUtil;
import org.enhydra.devsys.swing.GridBagConstraintsX;
import java.awt.*;
import java.beans.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * The Wizard dialog defines a default container for presenting
 * wizards. To use this class you need to a create wizard page
 * containing any swing elements unique to your wizard and then
 * add that page to this wizard dialog. This dialog supplies the
 * OK and Cancel buttons for closing the dialog.
 * <P><I>
 * The API used here modeled after that used in the JBuilder
 * Foundation open tools API.
 * </I></P>
 * TODO: Add support for multiple pages.
 *
 * @author
 * Paul Mahar
 * @version %I%, %G%
 */
public class WizardDialog extends JDialog {
    public boolean		standalone = false;
    public final static int     CHOICE_OK = 0;
    public final static int     CHOICE_CANCEL = 1;
    private int			choice = WizardDialog.CHOICE_CANCEL;
    private JPanel		panelPage;
    private JPanel		panelLine;
    private JPanel		panelConstant;
    private JPanel		panelMain;
    private GridBagLayout       layoutMain;
    private Border		border;
    private GridBagLayout       layoutConstant;
    private JButton		buttonOK;
    private JButton		buttonCancel;
    private JPanel		panelFiller;
    private BorderLayout	layoutPage;
    private LocalButtonListener buttonListener;

    /**
     * Create a wizard dialog as a child of another dialog.
     *
     * @param owner
     * The dialog that is the parent of this wizard.
     * @param title
     * Title to show in the caption bar.
     * @param modal
     * Pass true to make the dialog modal. Wizards are almost
     * always modal.
     *
     */
    public WizardDialog(Dialog owner, String title, boolean modal) {
	super(owner, title, modal);

	construct();
    }

    /**
     * Create a wizard dialog as a child of a frame.
     *
     * @param owner
     * The frame that is the parent of this wizard.
     * @param title
     * Title to show in the caption bar.
     * @param modal
     * Pass true to make the dialog modal. Wizards are almost
     * always modal.
     *
     */
    public WizardDialog(Frame owner, String title, boolean modal) {
	super(owner, title, modal);

	construct();
    }

    /**
     * Clear all object references.
     */
    public void clearAll() {
	this.removeAll();
	this.dispose();
    }

    /**
     * Get the button choice that was used to close the dialog.
     *
     * @return
     * CHOICE_OK or CHOICE_CANCEL
     *
     */
    public int getChoice() {
	return choice;
    }

    /**
     * Get the title used for the caption bar.
     */
    public String getWizardTitle() {
	return this.getTitle();
    }

    /**
     * Set the title to use in the caption bar.
     */
    public void setWizardTitle(String s) {
	this.setTitle(s);
    }

    /**
     * Add a page to the wizard.
     * <P>
     * TODO: Add support for more than one page.
     * </P>
     *
     * @param page
     * A panel that contains swing controls that
     * are specific to the current wizard.
     */
    public void addWizardPage(WizardPage page) {
	panelPage.add(page, BorderLayout.CENTER);
    }

    /**
     * Overrides super.show()
     * <P>
     * Open the dialog in the center of the screen.
     * </p>
     */
    public void show() {
     // setResizable(false); // crashes JVM on Win98 (NT/Win95 OK).    	

	Point cPoint;

	invalidate();
	doLayout();
	pack();

	cPoint = SwingUtil.getCenteringPoint(getSize());

	setLocation(cPoint);
	super.show();
    }

    /**
     * Close the dialog and exit the VM if this wizard
     * was run as a standalone application.
     */
    protected void closeDialog() {
	setVisible(false);
	clearAll();

	if (standalone) {
	    System.exit(0);
	}
    }

    /**
     * Construct the object for either constructor.
     */
    private void construct() {
	try {
	    jbInit();
	    pmInit();
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    /**
     * Add listeners and other items that are not generated by
     * the JBuilder designer.
     */
    private void pmInit() {
	buttonListener = new LocalButtonListener();

	buttonOK.addActionListener(buttonListener);
	buttonCancel.addActionListener(buttonListener);
    }

    /**
     * UI code generated by JBuilder.
     * <P><I>Do not modify the signature of this method.</I></P>
     */
    private void jbInit() throws Exception {
	panelPage = (JPanel) Beans.instantiate(getClass().getClassLoader(),
					       JPanel.class.getName());
	panelConstant =
	    (JPanel) Beans.instantiate(getClass().getClassLoader(),
				       JPanel.class.getName());
	panelLine = (JPanel) Beans.instantiate(getClass().getClassLoader(),
					       JPanel.class.getName());
	panelMain = (JPanel) Beans.instantiate(getClass().getClassLoader(),
					       JPanel.class.getName());
	layoutMain =
	    (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
					      GridBagLayout.class.getName());
	border = BorderFactory.createEtchedBorder();
	layoutConstant =
	    (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
					      GridBagLayout.class.getName());
	buttonOK = (JButton) Beans.instantiate(getClass().getClassLoader(),
					       JButton.class.getName());
	buttonCancel =
	    (JButton) Beans.instantiate(getClass().getClassLoader(),
					JButton.class.getName());
	panelFiller = (JPanel) Beans.instantiate(getClass().getClassLoader(),
						 JPanel.class.getName());
	layoutPage =
	    (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
					     BorderLayout.class.getName());

	panelMain.setLayout(layoutMain);
	panelLine.setBorder(border);
	panelLine.setMinimumSize(new Dimension(5, 5));
	panelLine.setPreferredSize(new Dimension(5, 5));
	panelConstant.setLayout(layoutConstant);
	buttonOK.setText("OK");
	buttonCancel.setText("Cancel");
	panelPage.setLayout(layoutPage);
	panelMain.setMinimumSize(new Dimension(600, 400));
	panelMain.setPreferredSize(new Dimension(600, 400));
	panelPage.setMinimumSize(new Dimension(600, 350));
	panelPage.setPreferredSize(new Dimension(600, 350));
	panelMain.add(panelPage,
		      new GridBagConstraintsX(0, 0, 1, 1, 0.9, 0.9,
					     GridBagConstraints.CENTER,
					     GridBagConstraints.BOTH,
					     new Insets(5, 5, 5, 5), 0, 0));
	panelMain.add(panelLine,
		      new GridBagConstraintsX(0, 1, 1, 1, 0.0, 0.0,
					     GridBagConstraints.CENTER,
					     GridBagConstraints.HORIZONTAL,
					     new Insets(5, 5, 5, 5), 0, 0));
	panelMain.add(panelConstant,
		      new GridBagConstraintsX(0, 2, 1, 1, 0.1, 0.1,
					     GridBagConstraints.CENTER,
					     GridBagConstraints.BOTH,
					     new Insets(5, 5, 5, 5), 0, 0));
	panelConstant.add(buttonOK,
			  new GridBagConstraintsX(1, 0, 1, 1, 0.0, 0.0,
						 GridBagConstraints.CENTER,
						 GridBagConstraints.NONE,
						 new Insets(5, 5, 5, 5), 0,
						 0));
	panelConstant.add(buttonCancel,
			  new GridBagConstraintsX(2, 0, 1, 1, 0.0, 0.0,
						 GridBagConstraints.CENTER,
						 GridBagConstraints.NONE,
						 new Insets(5, 5, 5, 5), 0,
						 0));
	panelConstant.add(panelFiller,
			  new GridBagConstraintsX(0, 0, 1, 1, 0.1, 0.1,
						 GridBagConstraints.CENTER,
						 GridBagConstraints.BOTH,
						 new Insets(0, 0, 0, 0), 0,
						 0));
	this.getContentPane().add(panelMain);
    }

    /**
     * This inner class defines the actions for buttons in the wizard.
     *
     * @author
     * Paul Mahar
     * @version %I%, %G%
     */
    class LocalButtonListener implements ActionListener {

	/**
	 * ActionListener implementation.
	 *
	 * @param event
	 * Event that triggered this listener.
	 *
	 */
	public void actionPerformed(ActionEvent event) {
	    Object source = event.getSource();

	    if (source == buttonOK) {
		choice = WizardDialog.CHOICE_OK;

		closeDialog();
	    } else if (source == buttonCancel) {
		choice = WizardDialog.CHOICE_CANCEL;

		closeDialog();
	    }
	}

    }

}



/*--- formatting done in "Sun Java Convention" style on 01-16-2000 ---*/

