/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: AbstractBuildThread.java,v 1.1.2.2 2000/03/21 09:47:40 pmahar Exp $
 */

package org.enhydra.ide.jbuilder.common.wizard;

import org.enhydra.ide.jbuilder.common.node.OtterProject;
import org.enhydra.ide.jbuilder.common.node.OtterSelectionNode;
import org.enhydra.ide.jbuilder.common.capture.Writer;
import org.enhydra.ide.jbuilder.common.capture.WriteListener;

import java.io.PrintWriter;
import java.io.File;

/**
 * CompileThread calls XMLC once for each selected HTML file.
 *
 *
 * @author
 * Paul Mahar
 * @version %I%, %G%
 */
abstract public class AbstractBuildThread extends Thread {
    private boolean       printProgress = true;
    private boolean       fresh = true;
    private OtterProject  project = null;
    private OtterSelectionNode[]  nodes = null;
    private PrintWriter      traceWriter = null;

    /**
     * Create a CompileThread with listeners ready to
     * recieve output from XMLC.
     *
     *
     * @param listeners
     * A listener implements an onWrite method that is called
     * whenever XMLC writes output. The writes are captured
     * for display in a wizard.
     */

    public AbstractBuildThread() {
      this(new WriteListener[0]);
    }

    public AbstractBuildThread(WriteListener[] listeners) {
      this(listeners, false);
    }

    public AbstractBuildThread(WriteListener[] listeners,
                         boolean buffer) {
	Writer capture = new Writer();
        capture.setLineBuffered(buffer);

	for (int i = 0; i < listeners.length; i++) {
	    capture.addWriteListener(listeners[i]);
	}

	traceWriter = new PrintWriter(capture, false);
  }



   /**
     * Loop through all selected nodes files and build them.
     * Write out exceptions to any registered listeners.
     */
    abstract public void make();

    /**
     * Determine what files were created and write out the
     * build results to any registered listeners.
     */
    abstract public void printResults();

    /**
     * Get the HTML source files that are to be compiled.
     */
    public OtterSelectionNode[] getNodes() {
	return nodes;
    }

    /**
     * Set the HTML source files to compile.
     */
    public void setNodes(OtterSelectionNode[] n) {
	nodes = n;
    }

    /**
     * Get the IDE project that contains the HTML source files to compile.
     */
    public OtterProject getProject() {
	return project;
    }

    /**
     * Set the IDE project that contains the HTML source files to compile.
     */
    public void setProject(OtterProject bp) {
	project = bp;
    }

    public boolean isPrintProgress() {
      return printProgress;
    }

    public void setPrintProgress(boolean b) {
      printProgress = b;
    }

    public PrintWriter getTraceWriter() {
      return traceWriter;
    }



    /**
     * Run the XMLC for all selected HTML files.
     */
    public void run() {
	fresh = true;
	make();
    }

    public synchronized void runInCurrentThread() {
	fresh = true;
	make();
    }

    /**
     * Make the thread stale so it stops running gracefully.
     */
    public void expire() {
	fresh = false;
    }

    public boolean isFresh() {
      return fresh;
    }


    public void printExceptions() {
      	int eCount = 0;
        int totalCount = getNodes().length;
      	for (int i = 0; i < totalCount; i++) {
	    if (getNodes()[i].isSelected() &&
                getNodes()[i].getException() != null) {
		eCount++;
	    }
	}
	if (eCount > 0) {
	    traceWriter.println("");
	    traceWriter.println("Exception encountered:");

	    for (int i = 0; i < totalCount; i++) {
		if (getNodes()[i].getException() != null) {
		    traceWriter.print("    ");
		    traceWriter.print(getNodes()[i].getFilename());
		    traceWriter.println(":");
		    traceWriter.println("    "
					+ getNodes()[i].getException().getMessage());
		}
	    }
	}
    }

    /**
     * Sleep the thread for a given number of seconds.
     *
     *
     * @param seconds
     * How many seconds to sleep.
     */
    public void sleepInSeconds(double seconds) {
	try {
	    Thread.sleep((int) (seconds * 1000));
	} catch (InterruptedException e) {}
    }

}
